from contextlib import contextmanager

from django.test.utils import modify_settings

from corsheaders.signals import check_request_enabled


def add_middleware(action, path):
    return modify_settings(**{"MIDDLEWARE": {action: path}})


def append_middleware(path):
    return add_middleware("append", path)


def prepend_middleware(path):
    return add_middleware("prepend", path)


@contextmanager
def temporary_check_request_hander(handler):
    check_request_enabled.connect(handler)
    try:
        yield
    finally:
        check_request_enabled.disconnect(handler)



from django.http import HttpResponse
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.deprecation import MiddlewareMixin

from corsheaders.middleware import (
    ACCESS_CONTROL_ALLOW_CREDENTIALS,
    ACCESS_CONTROL_ALLOW_HEADERS,
    ACCESS_CONTROL_ALLOW_METHODS,
    ACCESS_CONTROL_ALLOW_ORIGIN,
    ACCESS_CONTROL_EXPOSE_HEADERS,
    ACCESS_CONTROL_MAX_AGE,
)
from tests.utils import (
    append_middleware,
    prepend_middleware,
    temporary_check_request_hander,
)


class ShortCircuitMiddleware(MiddlewareMixin):
    def process_request(self, request):
        return HttpResponse("short-circuit-middleware-response")


class CorsMiddlewareTests(TestCase):
    def test_get_no_origin(self):
        resp = self.client.get("/")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_get_origin_vary_by_default(self):
        resp = self.client.get("/")
        assert resp["Vary"] == "Origin"

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    def test_get_not_in_allowed_origins(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.org")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(CORS_ALLOWED_ORIGINS=["https://example.org"])
    def test_get_not_in_allowed_origins_due_to_wrong_scheme(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.org")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://example.com", "http://example.org"]
    )
    def test_get_in_allowed_origins(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.org")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.org"

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com", "null"])
    def test_null_in_allowed_origins(self):
        resp = self.client.get("/", HTTP_ORIGIN="null")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "null"

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com", "file://"])
    def test_file_in_allowed_origins(self):
        """
        'file://' should be allowed as an origin since Chrome on Android
        mistakenly sends it
        """
        resp = self.client.get("/", HTTP_ORIGIN="file://")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "file://"

    @override_settings(
        CORS_ALLOW_ALL_ORIGINS=True,
        CORS_EXPOSE_HEADERS=["accept", "origin", "content-type"],
    )
    def test_get_expose_headers(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert resp[ACCESS_CONTROL_EXPOSE_HEADERS] == "accept, origin, content-type"

    @override_settings(CORS_ALLOW_ALL_ORIGINS=True)
    def test_get_dont_expose_headers(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_EXPOSE_HEADERS not in resp

    @override_settings(CORS_ALLOW_CREDENTIALS=True, CORS_ALLOW_ALL_ORIGINS=True)
    def test_get_allow_credentials(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert resp[ACCESS_CONTROL_ALLOW_CREDENTIALS] == "true"

    @override_settings(CORS_ALLOW_ALL_ORIGINS=True)
    def test_get_dont_allow_credentials(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_CREDENTIALS not in resp

    @override_settings(
        CORS_ALLOW_HEADERS=["content-type", "origin"],
        CORS_ALLOW_METHODS=["GET", "OPTIONS"],
        CORS_PREFLIGHT_MAX_AGE=1002,
        CORS_ALLOW_ALL_ORIGINS=True,
    )
    def test_options_allowed_origin(self):
        resp = self.client.options("/", HTTP_ORIGIN="http://example.com")
        assert resp[ACCESS_CONTROL_ALLOW_HEADERS] == "content-type, origin"
        assert resp[ACCESS_CONTROL_ALLOW_METHODS] == "GET, OPTIONS"
        assert resp[ACCESS_CONTROL_MAX_AGE] == "1002"

    @override_settings(
        CORS_ALLOW_HEADERS=["content-type", "origin"],
        CORS_ALLOW_METHODS=["GET", "OPTIONS"],
        CORS_PREFLIGHT_MAX_AGE=0,
        CORS_ALLOW_ALL_ORIGINS=True,
    )
    def test_options_no_max_age(self):
        resp = self.client.options("/", HTTP_ORIGIN="http://example.com")
        assert resp[ACCESS_CONTROL_ALLOW_HEADERS] == "content-type, origin"
        assert resp[ACCESS_CONTROL_ALLOW_METHODS] == "GET, OPTIONS"
        assert ACCESS_CONTROL_MAX_AGE not in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://localhost:9000"],
    )
    def test_options_allowed_origins_with_port(self):
        resp = self.client.options("/", HTTP_ORIGIN="http://localhost:9000")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://localhost:9000"

    @override_settings(
        CORS_ALLOWED_ORIGIN_REGEXES=[r"^https://\w+\.example\.com$"],
    )
    def test_options_adds_origin_when_domain_found_in_allowed_regexes(self):
        resp = self.client.options("/", HTTP_ORIGIN="https://foo.example.com")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "https://foo.example.com"

    @override_settings(
        CORS_ALLOWED_ORIGIN_REGEXES=[
            r"^https://\w+\.example\.org$",
            r"^https://\w+\.example\.com$",
        ],
    )
    def test_options_adds_origin_when_domain_found_in_allowed_regexes_second(self):
        resp = self.client.options("/", HTTP_ORIGIN="https://foo.example.com")
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "https://foo.example.com"

    @override_settings(
        CORS_ALLOWED_ORIGIN_REGEXES=[r"^https://\w+\.example\.org$"],
    )
    def test_options_doesnt_add_origin_when_domain_not_found_in_allowed_regexes(
        self,
    ):
        resp = self.client.options("/", HTTP_ORIGIN="https://foo.example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_options(self):
        resp = self.client.options("/", HTTP_ACCESS_CONTROL_REQUEST_METHOD="value")
        assert resp.status_code == 200

    def test_options_empty_request_method(self):
        resp = self.client.options("/", HTTP_ACCESS_CONTROL_REQUEST_METHOD="")
        assert resp.status_code == 200

    def test_options_no_header(self):
        resp = self.client.options("/")
        assert resp.status_code == 404

    @override_settings(CORS_ALLOW_CREDENTIALS=True, CORS_ALLOW_ALL_ORIGINS=True)
    def test_allow_all_origins_get(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.com"
        assert resp["Vary"] == "Origin"

    @override_settings(CORS_ALLOW_CREDENTIALS=True, CORS_ALLOW_ALL_ORIGINS=True)
    def test_allow_all_origins_options(self):
        resp = self.client.options(
            "/",
            HTTP_ORIGIN="http://example.com",
            HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
        )
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.com"
        assert resp["Vary"] == "Origin"

    @override_settings(CORS_ALLOW_CREDENTIALS=True, CORS_ALLOW_ALL_ORIGINS=True)
    def test_non_200_headers_still_set(self):
        """
        It's not clear whether the header should still be set for non-HTTP200
        when not a preflight request. However this is the existing behaviour for
        django-cors-middleware, so at least this test makes that explicit, especially
        since for the switch to Django 1.10, special-handling will need to be put in
        place to preserve this behaviour. See `ExceptionMiddleware` mention here:
        https://docs.djangoproject.com/en/3.0/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware  # noqa: B950
        """
        resp = self.client.get("/test-401/", HTTP_ORIGIN="http://example.com")
        assert resp.status_code == 401
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.com"

    @override_settings(CORS_ALLOW_CREDENTIALS=True, CORS_ALLOW_ALL_ORIGINS=True)
    def test_auth_view_options(self):
        """
        Ensure HTTP200 and header still set, for preflight requests to views requiring
        authentication. See: https://github.com/adamchainz/django-cors-headers/issues/3
        """
        resp = self.client.options(
            "/test-401/",
            HTTP_ORIGIN="http://example.com",
            HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
        )
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.com"
        assert resp["Content-Length"] == "0"

    def test_signal_handler_that_returns_false(self):
        def handler(*args, **kwargs):
            return False

        with temporary_check_request_hander(handler):
            resp = self.client.options(
                "/",
                HTTP_ORIGIN="http://example.com",
                HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
            )

            assert resp.status_code == 200
            assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_signal_handler_that_returns_true(self):
        def handler(*args, **kwargs):
            return True

        with temporary_check_request_hander(handler):
            resp = self.client.options(
                "/",
                HTTP_ORIGIN="http://example.com",
                HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
            )
            assert resp.status_code == 200
            assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.com"

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    def test_signal_handler_allow_some_urls_to_everyone(self):
        def allow_api_to_all(sender, request, **kwargs):
            return request.path.startswith("/api/")

        with temporary_check_request_hander(allow_api_to_all):
            resp = self.client.options(
                "/",
                HTTP_ORIGIN="http://example.org",
                HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
            )
            assert resp.status_code == 200
            assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

            resp = self.client.options(
                "/api/something/",
                HTTP_ORIGIN="http://example.org",
                HTTP_ACCESS_CONTROL_REQUEST_METHOD="value",
            )
            assert resp.status_code == 200
            assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == "http://example.org"

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    def test_signal_called_once_during_normal_flow(self):
        def allow_all(sender, request, **kwargs):
            allow_all.calls += 1
            return True

        allow_all.calls = 0

        with temporary_check_request_hander(allow_all):
            self.client.get("/", HTTP_ORIGIN="http://example.org")

            assert allow_all.calls == 1

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    @prepend_middleware("tests.test_middleware.ShortCircuitMiddleware")
    def test_get_short_circuit(self):
        """
        Test a scenario when a middleware that returns a response is run before
        the ``CorsMiddleware``. In this case
        ``CorsMiddleware.process_response()`` should ignore the request if
        MIDDLEWARE setting is used (new mechanism in Django 1.10+).
        """
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://example.com"], CORS_URLS_REGEX=r"^/foo/$"
    )
    @prepend_middleware(__name__ + ".ShortCircuitMiddleware")
    def test_get_short_circuit_should_be_ignored(self):
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://example.com"], CORS_URLS_REGEX=r"^/foo/$"
    )
    def test_get_regex_matches(self):
        resp = self.client.get("/foo/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://example.com"], CORS_URLS_REGEX=r"^/not-foo/$"
    )
    def test_get_regex_doesnt_match(self):
        resp = self.client.get("/foo/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ALLOWED_ORIGINS=["http://example.com"], CORS_URLS_REGEX=r"^/foo/$"
    )
    def test_get_regex_matches_path_info(self):
        resp = self.client.get(
            "/foo/", HTTP_ORIGIN="http://example.com", SCRIPT_NAME="/prefix/"
        )
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    def test_cors_enabled_is_attached_and_bool(self):
        """
        Ensure that request._cors_enabled is available - although a private API
        someone might use it for debugging
        """
        resp = self.client.get("/", HTTP_ORIGIN="http://example.com")
        request = resp.wsgi_request
        assert isinstance(request._cors_enabled, bool)
        assert request._cors_enabled

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com"])
    def test_works_if_view_deletes_cors_enabled(self):
        """
        Just in case something crazy happens in the view or other middleware,
        check that get_response doesn't fall over if `_cors_enabled` is removed
        """
        resp = self.client.get("/delete-is-enabled/", HTTP_ORIGIN="http://example.com")
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp


@override_settings(
    CORS_REPLACE_HTTPS_REFERER=True, CORS_ALLOWED_ORIGIN_REGEXES=[r".*example.*"]
)
class RefererReplacementCorsMiddlewareTests(TestCase):
    def test_get_replaces_referer_when_secure(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.com/"
        assert (
            resp.wsgi_request.META["ORIGINAL_HTTP_REFERER"] == "https://example.org/foo"
        )

    @append_middleware("corsheaders.middleware.CorsPostCsrfMiddleware")
    def test_get_post_middleware_rereplaces_referer_when_secure(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.org/foo"
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META

    def test_get_does_not_replace_referer_when_insecure(self):
        resp = self.client.get(
            "/",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.org/foo"
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META

    @override_settings(CORS_REPLACE_HTTPS_REFERER=False)
    def test_get_does_not_replace_referer_when_disabled(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.org/foo"
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META

    def test_get_does_not_fail_in_referer_replacement_when_referer_missing(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
        )
        assert resp.status_code == 200
        assert "HTTP_REFERER" not in resp.wsgi_request.META
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META

    def test_get_does_not_fail_in_referer_replacement_when_host_missing(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.org/foo"
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META

    @override_settings(CORS_ALLOWED_ORIGIN_REGEXES=[])
    def test_get_does_not_replace_referer_when_not_valid_request(self):
        resp = self.client.get(
            "/",
            HTTP_FAKE_SECURE="true",
            HTTP_HOST="example.com",
            HTTP_ORIGIN="https://example.org",
            HTTP_REFERER="https://example.org/foo",
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META["HTTP_REFERER"] == "https://example.org/foo"
        assert "ORIGINAL_HTTP_REFERER" not in resp.wsgi_request.META



import re
from collections.abc import Sequence
from numbers import Integral
from urllib.parse import urlparse

from django.conf import settings
from django.core import checks

from corsheaders.conf import conf

re_type = type(re.compile(""))


def check_settings(app_configs, **kwargs):
    errors = []

    if not is_sequence(conf.CORS_ALLOW_HEADERS, str):
        errors.append(
            checks.Error(
                "CORS_ALLOW_HEADERS should be a sequence of strings.",
                id="corsheaders.E001",
            )
        )

    if not is_sequence(conf.CORS_ALLOW_METHODS, str):
        errors.append(
            checks.Error(
                "CORS_ALLOW_METHODS should be a sequence of strings.",
                id="corsheaders.E002",
            )
        )

    if not isinstance(conf.CORS_ALLOW_CREDENTIALS, bool):
        errors.append(
            checks.Error(
                "CORS_ALLOW_CREDENTIALS should be a bool.", id="corsheaders.E003"
            )
        )

    if (
        not isinstance(conf.CORS_PREFLIGHT_MAX_AGE, Integral)
        or conf.CORS_PREFLIGHT_MAX_AGE < 0
    ):
        errors.append(
            checks.Error(
                (
                    "CORS_PREFLIGHT_MAX_AGE should be an integer greater than "
                    + "or equal to zero."
                ),
                id="corsheaders.E004",
            )
        )

    if not isinstance(conf.CORS_ALLOW_ALL_ORIGINS, bool):
        if hasattr(settings, "CORS_ALLOW_ALL_ORIGINS"):
            allow_all_alias = "CORS_ALLOW_ALL_ORIGINS"
        else:
            allow_all_alias = "CORS_ORIGIN_ALLOW_ALL"
        errors.append(
            checks.Error(
                "{} should be a bool.".format(allow_all_alias),
                id="corsheaders.E005",
            )
        )

    if hasattr(settings, "CORS_ALLOWED_ORIGINS"):
        allowed_origins_alias = "CORS_ALLOWED_ORIGINS"
    else:
        allowed_origins_alias = "CORS_ORIGIN_WHITELIST"

    if not is_sequence(conf.CORS_ALLOWED_ORIGINS, str):
        errors.append(
            checks.Error(
                "{} should be a sequence of strings.".format(allowed_origins_alias),
                id="corsheaders.E006",
            )
        )
    else:
        special_origin_values = (
            # From 'security sensitive' contexts
            "null",
            # From files on Chrome on Android
            # https://bugs.chromium.org/p/chromium/issues/detail?id=991107
            "file://",
        )
        for origin in conf.CORS_ALLOWED_ORIGINS:
            if origin in special_origin_values:
                continue
            parsed = urlparse(origin)
            if parsed.scheme == "" or parsed.netloc == "":
                errors.append(
                    checks.Error(
                        "Origin {} in {} is missing scheme or netloc".format(
                            repr(origin), allowed_origins_alias
                        ),
                        id="corsheaders.E013",
                        hint=(
                            "Add a scheme (e.g. https://) or netloc (e.g. "
                            + "example.com)."
                        ),
                    )
                )
            else:
                # Only do this check in this case because if the scheme is not
                # provided, netloc ends up in path
                for part in ("path", "params", "query", "fragment"):
                    if getattr(parsed, part) != "":
                        errors.append(
                            checks.Error(
                                "Origin {} in {} should not have {}".format(
                                    repr(origin), allowed_origins_alias, part
                                ),
                                id="corsheaders.E014",
                            )
                        )

    if hasattr(settings, "CORS_ALLOWED_ORIGIN_REGEXES"):
        allowed_regexes_alias = "CORS_ALLOWED_ORIGIN_REGEXES"
    else:
        allowed_regexes_alias = "CORS_ORIGIN_REGEX_WHITELIST"
    if not is_sequence(conf.CORS_ALLOWED_ORIGIN_REGEXES, (str, re_type)):
        errors.append(
            checks.Error(
                "{} should be a sequence of strings and/or compiled regexes.".format(
                    allowed_regexes_alias
                ),
                id="corsheaders.E007",
            )
        )

    if not is_sequence(conf.CORS_EXPOSE_HEADERS, str):
        errors.append(
            checks.Error(
                "CORS_EXPOSE_HEADERS should be a sequence.", id="corsheaders.E008"
            )
        )

    if not isinstance(conf.CORS_URLS_REGEX, (str, re_type)):
        errors.append(
            checks.Error(
                "CORS_URLS_REGEX should be a string or regex.", id="corsheaders.E009"
            )
        )

    if not isinstance(conf.CORS_REPLACE_HTTPS_REFERER, bool):
        errors.append(
            checks.Error(
                "CORS_REPLACE_HTTPS_REFERER should be a bool.", id="corsheaders.E011"
            )
        )

    if hasattr(settings, "CORS_MODEL"):
        errors.append(
            checks.Error(
                (
                    "The CORS_MODEL setting has been removed - see "
                    + "django-cors-headers' HISTORY."
                ),
                id="corsheaders.E012",
            )
        )

    return errors


def is_sequence(thing, type_or_types):
    return isinstance(thing, Sequence) and all(
        isinstance(x, type_or_types) for x in thing
    )



from django.conf import settings

# Kept here for backwards compatibility
from corsheaders.defaults import default_headers, default_methods


class Settings:
    """
    Shadow Django's settings with a little logic
    """

    @property
    def CORS_ALLOW_HEADERS(self):
        return getattr(settings, "CORS_ALLOW_HEADERS", default_headers)

    @property
    def CORS_ALLOW_METHODS(self):
        return getattr(settings, "CORS_ALLOW_METHODS", default_methods)

    @property
    def CORS_ALLOW_CREDENTIALS(self):
        return getattr(settings, "CORS_ALLOW_CREDENTIALS", False)

    @property
    def CORS_PREFLIGHT_MAX_AGE(self):
        return getattr(settings, "CORS_PREFLIGHT_MAX_AGE", 86400)

    @property
    def CORS_ALLOW_ALL_ORIGINS(self):
        return getattr(
            settings,
            "CORS_ALLOW_ALL_ORIGINS",
            getattr(settings, "CORS_ORIGIN_ALLOW_ALL", False),
        )

    @property
    def CORS_ALLOWED_ORIGINS(self):
        return getattr(
            settings,
            "CORS_ALLOWED_ORIGINS",
            getattr(settings, "CORS_ORIGIN_WHITELIST", ()),
        )

    @property
    def CORS_ALLOWED_ORIGIN_REGEXES(self):
        return getattr(
            settings,
            "CORS_ALLOWED_ORIGIN_REGEXES",
            getattr(settings, "CORS_ORIGIN_REGEX_WHITELIST", ()),
        )

    @property
    def CORS_EXPOSE_HEADERS(self):
        return getattr(settings, "CORS_EXPOSE_HEADERS", ())

    @property
    def CORS_URLS_REGEX(self):
        return getattr(settings, "CORS_URLS_REGEX", r"^.*$")

    @property
    def CORS_REPLACE_HTTPS_REFERER(self):
        return getattr(settings, "CORS_REPLACE_HTTPS_REFERER", False)


conf = Settings()



import re

import pytest
from django.core.checks import Error
from django.core.management import base, call_command
from django.test import SimpleTestCase
from django.test.utils import override_settings

from corsheaders.checks import check_settings


class ChecksTests(SimpleTestCase):
    def check_error_codes(self, expected):
        errors = check_settings([])
        assert len(errors) == len(expected)
        assert all(isinstance(e, Error) for e in errors)
        assert [e.id for e in errors] == expected
        return errors

    def test_defaults_pass(self):
        self.check_error_codes([])

    def test_defaults_pass_check(self):
        call_command("check")

    @override_settings(CORS_ALLOW_ALL_ORIGINS=object)
    def test_checks_are_bound(self):
        with pytest.raises(base.SystemCheckError):
            call_command("check")

    @override_settings(CORS_ALLOW_HEADERS=object)
    def test_cors_allow_headers_non_sequence(self):
        self.check_error_codes(["corsheaders.E001"])

    @override_settings(CORS_ALLOW_HEADERS=[object])
    def test_cors_allow_headers_non_string(self):
        self.check_error_codes(["corsheaders.E001"])

    @override_settings(CORS_ALLOW_METHODS=object)
    def test_cors_allow_methods_non_sequence(self):
        self.check_error_codes(["corsheaders.E002"])

    @override_settings(CORS_ALLOW_METHODS=[object])
    def test_cors_allow_methods_non_string(self):
        self.check_error_codes(["corsheaders.E002"])

    @override_settings(CORS_ALLOW_CREDENTIALS=object)
    def test_cors_allow_credentials_non_bool(self):
        self.check_error_codes(["corsheaders.E003"])

    @override_settings(CORS_PREFLIGHT_MAX_AGE="10")
    def test_cors_preflight_max_age_non_integer(self):
        self.check_error_codes(["corsheaders.E004"])

    @override_settings(CORS_PREFLIGHT_MAX_AGE=-1)
    def test_cors_preflight_max_age_negative(self):
        self.check_error_codes(["corsheaders.E004"])

    @override_settings(CORS_ALLOW_ALL_ORIGINS=object)
    def test_cors_allow_all_origins_non_bool(self):
        errors = self.check_error_codes(["corsheaders.E005"])
        assert errors[0].msg.startswith("CORS_ALLOW_ALL_ORIGINS should be")

    @override_settings(CORS_ORIGIN_ALLOW_ALL=object)
    def test_cors_allow_all_origins_old_name(self):
        errors = self.check_error_codes(["corsheaders.E005"])
        assert errors[0].msg.startswith("CORS_ORIGIN_ALLOW_ALL should be")

    @override_settings(CORS_ALLOWED_ORIGINS=object)
    def test_cors_allowed_origins_non_sequence(self):
        self.check_error_codes(["corsheaders.E006"])

    @override_settings(CORS_ALLOWED_ORIGINS=[object])
    def test_cors_allowed_origins_non_string(self):
        self.check_error_codes(["corsheaders.E006"])

    @override_settings(CORS_ORIGIN_WHITELIST=object)
    def test_cors_allowed_origins_old_name(self):
        errors = self.check_error_codes(["corsheaders.E006"])
        assert errors[0].msg.startswith("CORS_ORIGIN_WHITELIST should be")

    @override_settings(CORS_ALLOWED_ORIGINS=["http://example.com", "file://", "null"])
    def test_cors_allowed_origins_allowed(self):
        self.check_error_codes([])

    @override_settings(CORS_ALLOWED_ORIGINS=["example.com"])
    def test_cors_allowed_origins_no_scheme(self):
        errors = self.check_error_codes(["corsheaders.E013"])
        assert "in CORS_ALLOWED_ORIGINS" in errors[0].msg

    @override_settings(CORS_ORIGIN_WHITELIST=["example.com"])
    def test_cors_allowed_origins_no_scheme_old_name(self):
        errors = self.check_error_codes(["corsheaders.E013"])
        assert "in CORS_ORIGIN_WHITELIST" in errors[0].msg

    @override_settings(CORS_ALLOWED_ORIGINS=["https://"])
    def test_cors_allowed_origins_no_netloc(self):
        self.check_error_codes(["corsheaders.E013"])

    @override_settings(CORS_ALLOWED_ORIGINS=["https://example.com/foobar"])
    def test_cors_allowed_origins_path(self):
        errors = self.check_error_codes(["corsheaders.E014"])
        assert "in CORS_ALLOWED_ORIGINS" in errors[0].msg

    @override_settings(CORS_ORIGIN_WHITELIST=["https://example.com/foobar"])
    def test_cors_allowed_origins_path_old_name(self):
        errors = self.check_error_codes(["corsheaders.E014"])
        assert "in CORS_ORIGIN_WHITELIST" in errors[0].msg

    @override_settings(CORS_ALLOWED_ORIGIN_REGEXES=object)
    def test_cors_allowed_origin_regexes_non_sequence(self):
        self.check_error_codes(["corsheaders.E007"])

    @override_settings(CORS_ORIGIN_REGEX_WHITELIST=object)
    def test_cors_allowed_origin_regexes_old_name(self):
        errors = self.check_error_codes(["corsheaders.E007"])
        assert errors[0].msg.startswith("CORS_ORIGIN_REGEX_WHITELIST should be")

    @override_settings(CORS_ALLOWED_ORIGIN_REGEXES=[re.compile(r"a")])
    def test_cors_allowed_origin_regexes_regex(self):
        self.check_error_codes([])

    @override_settings(CORS_EXPOSE_HEADERS=object)
    def test_cors_expose_headers_non_sequence(self):
        self.check_error_codes(["corsheaders.E008"])

    @override_settings(CORS_EXPOSE_HEADERS=[object])
    def test_cors_expose_headers_non_string(self):
        self.check_error_codes(["corsheaders.E008"])

    @override_settings(CORS_URLS_REGEX=object)
    def test_cors_urls_regex_non_string(self):
        self.check_error_codes(["corsheaders.E009"])

    @override_settings(CORS_REPLACE_HTTPS_REFERER=object)
    def test_cors_replace_https_referer_failure(self):
        self.check_error_codes(["corsheaders.E011"])

    @override_settings(CORS_MODEL="something")
    def test_cors_model_failure(self):
        self.check_error_codes(["corsheaders.E012"])



from django.dispatch import Signal

# If any attached handler returns Truthy, CORS will be allowed for the request.
# This can be used to build custom logic into the request handling when the
# configuration doesn't work.
check_request_enabled = Signal()



SECRET_KEY = "NOTASECRET"

ALLOWED_HOSTS = ["*"]

INSTALLED_APPS = ["corsheaders"]

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}

TEMPLATES = [{"BACKEND": "django.template.backends.django.DjangoTemplates"}]

ROOT_URLCONF = "tests.urls"

MIDDLEWARE = ["corsheaders.middleware.CorsMiddleware"]

SECURE_PROXY_SSL_HEADER = ("HTTP_FAKE_SECURE", "true")



from django.apps import AppConfig
from django.core.checks import Tags, register

from corsheaders.checks import check_settings


class CorsHeadersAppConfig(AppConfig):
    name = "corsheaders"
    verbose_name = "django-cors-headers"

    def ready(self):
        register(Tags.security)(check_settings)



from django.http import Http404, HttpResponse
from django.urls import path


def test_view(request):
    if request.method != "GET":
        raise Http404()
    return HttpResponse("Test view")


def test_view_http401(request):
    return HttpResponse("Unauthorized", status=401)


def test_view_that_deletes_is_enabled(request):
    if hasattr(request, "_cors_enabled"):
        del request._cors_enabled
    return HttpResponse()


urlpatterns = [
    path("", test_view),
    path("foo/", test_view),
    path("test-401/", test_view_http401),
    path("delete-is-enabled/", test_view_that_deletes_is_enabled),
]



from django.test import SimpleTestCase
from django.test.utils import override_settings

from corsheaders.conf import conf


class ConfTests(SimpleTestCase):
    @override_settings(CORS_ALLOW_HEADERS=["foo"])
    def test_can_override(self):
        assert conf.CORS_ALLOW_HEADERS == ["foo"]

    @override_settings(CORS_ORIGIN_ALLOW_ALL=True)
    def test_cors_allow_all_origins_old_alias(self):
        assert conf.CORS_ALLOW_ALL_ORIGINS is True

    @override_settings(
        CORS_ALLOW_ALL_ORIGINS=False,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_cors_allow_all_origins_new_setting_takes_precedence(self):
        assert conf.CORS_ALLOW_ALL_ORIGINS is False

    @override_settings(CORS_ORIGIN_WHITELIST=["example.com"])
    def test_cors_allowed_origins_old_alias(self):
        assert conf.CORS_ALLOWED_ORIGINS == ["example.com"]

    @override_settings(
        CORS_ALLOWED_ORIGINS=["example.com"], CORS_ORIGIN_WHITELIST=["example.org"]
    )
    def test_cors_allowed_origins_new_setting_takes_precedence(self):
        assert conf.CORS_ALLOWED_ORIGINS == ["example.com"]

    @override_settings(CORS_ORIGIN_REGEX_WHITELIST=[r".*"])
    def test_cors_allowed_origin_regexes_old_alias(self):
        assert conf.CORS_ALLOWED_ORIGIN_REGEXES == [".*"]

    @override_settings(
        CORS_ALLOWED_ORIGIN_REGEXES=["a+"], CORS_ORIGIN_REGEX_WHITELIST=[".*"]
    )
    def test_cors_allowed_origin_regexes_new_setting_takes_precedence(self):
        assert conf.CORS_ALLOWED_ORIGIN_REGEXES == ["a+"]



default_headers = (
    "accept",
    "accept-encoding",
    "authorization",
    "content-type",
    "dnt",
    "origin",
    "user-agent",
    "x-csrftoken",
    "x-requested-with",
)

default_methods = ("DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT")



import re
from urllib.parse import urlparse

from django import http
from django.utils.cache import patch_vary_headers
from django.utils.deprecation import MiddlewareMixin

from corsheaders.conf import conf
from corsheaders.signals import check_request_enabled

ACCESS_CONTROL_ALLOW_ORIGIN = "Access-Control-Allow-Origin"
ACCESS_CONTROL_EXPOSE_HEADERS = "Access-Control-Expose-Headers"
ACCESS_CONTROL_ALLOW_CREDENTIALS = "Access-Control-Allow-Credentials"
ACCESS_CONTROL_ALLOW_HEADERS = "Access-Control-Allow-Headers"
ACCESS_CONTROL_ALLOW_METHODS = "Access-Control-Allow-Methods"
ACCESS_CONTROL_MAX_AGE = "Access-Control-Max-Age"


class CorsPostCsrfMiddleware(MiddlewareMixin):
    def _https_referer_replace_reverse(self, request):
        """
        Put the HTTP_REFERER back to its original value and delete the
        temporary storage
        """
        if conf.CORS_REPLACE_HTTPS_REFERER and "ORIGINAL_HTTP_REFERER" in request.META:
            http_referer = request.META["ORIGINAL_HTTP_REFERER"]
            request.META["HTTP_REFERER"] = http_referer
            del request.META["ORIGINAL_HTTP_REFERER"]

    def process_request(self, request):
        self._https_referer_replace_reverse(request)
        return None

    def process_view(self, request, callback, callback_args, callback_kwargs):
        self._https_referer_replace_reverse(request)
        return None


class CorsMiddleware(MiddlewareMixin):
    def _https_referer_replace(self, request):
        """
        When https is enabled, django CSRF checking includes referer checking
        which breaks when using CORS. This function updates the HTTP_REFERER
        header to make sure it matches HTTP_HOST, provided that our cors logic
        succeeds
        """
        origin = request.META.get("HTTP_ORIGIN")

        if (
            request.is_secure()
            and origin
            and "ORIGINAL_HTTP_REFERER" not in request.META
        ):

            url = urlparse(origin)
            if (
                not conf.CORS_ALLOW_ALL_ORIGINS
                and not self.origin_found_in_white_lists(origin, url)
            ):
                return

            try:
                http_referer = request.META["HTTP_REFERER"]
                http_host = "https://%s/" % request.META["HTTP_HOST"]
                request.META = request.META.copy()
                request.META["ORIGINAL_HTTP_REFERER"] = http_referer
                request.META["HTTP_REFERER"] = http_host
            except KeyError:
                pass

    def process_request(self, request):
        """
        If CORS preflight header, then create an
        empty body response (200 OK) and return it

        Django won't bother calling any other request
        view/exception middleware along with the requested view;
        it will call any response middlewares
        """
        request._cors_enabled = self.is_enabled(request)
        if request._cors_enabled:
            if conf.CORS_REPLACE_HTTPS_REFERER:
                self._https_referer_replace(request)

            if (
                request.method == "OPTIONS"
                and "HTTP_ACCESS_CONTROL_REQUEST_METHOD" in request.META
            ):
                response = http.HttpResponse()
                response["Content-Length"] = "0"
                return response

    def process_view(self, request, callback, callback_args, callback_kwargs):
        """
        Do the referer replacement here as well
        """
        if request._cors_enabled and conf.CORS_REPLACE_HTTPS_REFERER:
            self._https_referer_replace(request)
        return None

    def process_response(self, request, response):
        """
        Add the respective CORS headers
        """
        enabled = getattr(request, "_cors_enabled", None)
        if enabled is None:
            enabled = self.is_enabled(request)

        if not enabled:
            return response

        patch_vary_headers(response, ["Origin"])

        origin = request.META.get("HTTP_ORIGIN")
        if not origin:
            return response

        # todo: check hostname from db instead
        url = urlparse(origin)

        if conf.CORS_ALLOW_CREDENTIALS:
            response[ACCESS_CONTROL_ALLOW_CREDENTIALS] = "true"

        if (
            not conf.CORS_ALLOW_ALL_ORIGINS
            and not self.origin_found_in_white_lists(origin, url)
            and not self.check_signal(request)
        ):
            return response

        if conf.CORS_ALLOW_ALL_ORIGINS and not conf.CORS_ALLOW_CREDENTIALS:
            response[ACCESS_CONTROL_ALLOW_ORIGIN] = "*"
        else:
            response[ACCESS_CONTROL_ALLOW_ORIGIN] = origin

        if len(conf.CORS_EXPOSE_HEADERS):
            response[ACCESS_CONTROL_EXPOSE_HEADERS] = ", ".join(
                conf.CORS_EXPOSE_HEADERS
            )

        if request.method == "OPTIONS":
            response[ACCESS_CONTROL_ALLOW_HEADERS] = ", ".join(conf.CORS_ALLOW_HEADERS)
            response[ACCESS_CONTROL_ALLOW_METHODS] = ", ".join(conf.CORS_ALLOW_METHODS)
            if conf.CORS_PREFLIGHT_MAX_AGE:
                response[ACCESS_CONTROL_MAX_AGE] = conf.CORS_PREFLIGHT_MAX_AGE

        return response

    def origin_found_in_white_lists(self, origin, url):
        return (
            (origin == "null" and origin in conf.CORS_ALLOWED_ORIGINS)
            or self._url_in_whitelist(url)
            or self.regex_domain_match(origin)
        )

    def regex_domain_match(self, origin):
        for domain_pattern in conf.CORS_ALLOWED_ORIGIN_REGEXES:
            if re.match(domain_pattern, origin):
                return origin

    def is_enabled(self, request):
        return bool(
            re.match(conf.CORS_URLS_REGEX, request.path_info)
        ) or self.check_signal(request)

    def check_signal(self, request):
        signal_responses = check_request_enabled.send(sender=None, request=request)
        return any(return_value for function, return_value in signal_responses)

    def _url_in_whitelist(self, url):
        origins = [urlparse(o) for o in conf.CORS_ALLOWED_ORIGINS]
        return any(
            origin.scheme == url.scheme and origin.netloc == url.netloc
            for origin in origins
        )
